import React, { useEffect, useRef, useState } from "react";
import "../../App.css";
import Footer from "../../Footer";
import InputField from "../InputField";
import SubmitField from "../SubmitField";
import { Logger } from "@behametrics/logger-web";
import { Button } from "../Button";
import BehaviorStudyInfo from "../BehaviorStudyInfo";
import BehaviorNormal from "../BehaviorNormal";
import BehaviorPhone from "../BehaviorPhone";
import BehaviorStanding from "../BehaviorStanding";
import BehaviorStudyEnd from "../BehaviorStudyEnd";
import { authFetch } from "../../auth/AuthProvider";

export default function Study() {
  const _logger = useRef(0);
  const [serverUsername, setServerUsername] = useState("");
  const [genPassword, setGenPassword] = useState("");
  const STATES = {
    START: "start",
    NORMAL: "normal",
    PHONE: "phone",
    STANDING: "standing",
    END: "end",
  };
  const [state, setState] = useState(STATES.START);

  useEffect(() => {
    _logger.current = new Logger({
      //inputs: ["cursor", "wheel", "keyboard", "touch"],
      inputs: ["keyboard"],
      // apiUrl: "https://behavior.marcocamenzind.ch",
      apiUrl: "http://localhost:5000",
      logToConsole: true,
    });
    _logger.current.init();
    authFetch("/api/username", {
      method: "get",
    }).then((response) => {
      response.json().then((resp) => {
        setServerUsername(resp.username);
      });
    });
  }, []);

  let username = "";
  const setUsername = (tmp_username) => {
    username = tmp_username;
  };

  let password = "";
  const setPassword = (tmp_password) => {
    password = tmp_password;
  };

  const handleLoggerOff = () => {
    _logger.current.stop();
    console.log("Logger ausgeschaltet");
  };

  const handleLoggerOn = () => {
    _logger.current.start();
    console.log("start logging ");
  };

  const handlePasswordChange = (e) => {
    setPassword(e.target.value);
  };

  const handleUsernameChange = (e) => {
    setUsername(e.target.value);
  };

  const handleOnPasteEvent = (e) => {
    e.preventDefault();
    console.log("paste not allowed");
    return false;
  };

  const checkIfUsernameIsCorrect = () => {
    return serverUsername === username;
  };

  const receiveRandomPassword = () => {
    fetch("/api/rcv_pw", {
      method: "get",
    }).then((response) => {
      response.json().then((resp) => {
        console.log(resp.random_password);
        setGenPassword(resp.random_password);
        console.log("rcv pw; print genPassword", genPassword);
      });
    });
  };

  const checkIfPasswordIsCorrect = () => {
    return genPassword === password;
  };

  const checkIfValuesAreCorrect = () => {
    console.log(checkIfPasswordIsCorrect());
    return checkIfPasswordIsCorrect() && checkIfUsernameIsCorrect();
  };

  const handleClickAtStepStart = () => {
    receiveRandomPassword();
    setState(STATES.NORMAL);
    handleLoggerOn();
  };

  const handleClickAtStepNormal = () => {
    if (checkIfValuesAreCorrect()) {
      receiveRandomPassword();
      setState(STATES.PHONE);
    }
  };

  const handleClickAtStepPhone = () => {
    if (checkIfValuesAreCorrect()) {
      receiveRandomPassword();
      setState(STATES.STANDING);
    } else {
      console.log("Passwort und Benutzername stimmen nicht.");
    }
  };

  const handleClickAtStepStanding = () => {
    if (checkIfValuesAreCorrect()) {
      setState(STATES.END);
      handleLoggerOff();
    } else {
      console.log("Passwort und Benutzername stimmen nicht.");
    }
  };

  const study_start = (
    <>
      <BehaviorStudyInfo />
      <Button
        className="btns"
        buttonStyle="btn--primary"
        buttonSize="btn--full"
        onClick={handleClickAtStepStart}
        newTo="study"
      >
        Studie starten
      </Button>
    </>
  );

  const study_normal = (
    <>
      <BehaviorNormal genPassword={genPassword} />
      <form id="behaviorNormal">
        <InputField
          LabelName="Benutzername"
          onChange={handleUsernameChange}
          InputType="text"
          InputName="Benutzername"
          InputPlaceHolder="Benutzername"
          onPaste={handleOnPasteEvent}
        />
        <InputField
          LabelName="Passwort"
          onChange={handlePasswordChange}
          InputType="password"
          InputName="Passwort"
          InputPlaceHolder="Benutzername"
          onPaste={handleOnPasteEvent}
        />
        <SubmitField
          LabelName="Weiter zur nächsten Situation"
          InputValue="next-situation"
          InputName="Weiter"
          onClick={handleClickAtStepNormal}
        />
      </form>
    </>
  );

  const study_phone = (
    <>
      <BehaviorPhone genPassword={genPassword} />
      <form id="behaviorPhone">
        <InputField
          LabelName="Benutzername"
          onChange={handleUsernameChange}
          InputType="text"
          InputName="Benutzername"
          InputPlaceHolder="Benutzername"
          onPaste={handleOnPasteEvent}
        />
        <InputField
          LabelName="Passwort"
          onChange={handlePasswordChange}
          InputType="password"
          InputName="Passwort"
          InputPlaceHolder="Benutzername"
          onPaste={handleOnPasteEvent}
        />
        <SubmitField
          LabelName="Weiter zur nächsten Situation"
          InputValue="next-situation"
          InputName="Weiter"
          onClick={handleClickAtStepPhone}
        />
      </form>
    </>
  );

  const study_standing = (
    <>
      <BehaviorStanding genPassword={genPassword} />
      <form id="behaviorStanding">
        <InputField
          LabelName="Benutzername"
          onChange={handleUsernameChange}
          InputType="text"
          InputName="Benutzername"
          InputPlaceHolder="Benutzername"
          onPaste={handleOnPasteEvent}
        />
        <InputField
          LabelName="Passwort"
          onChange={handlePasswordChange}
          InputType="password"
          InputName="Passwort"
          InputPlaceHolder="Passwort"
          onPaste={handleOnPasteEvent}
        />
        <SubmitField
          LabelName="Studie beenden"
          InputValue="stopStudy"
          InputName="quit"
          onClick={handleClickAtStepStanding}
        />
      </form>
    </>
  );

  const study_end = <BehaviorStudyEnd />;

  return (
    <>
      <div className="sitePage">
        <h1>Studie</h1>
        {state === STATES.START ? study_start : null}
        {state === STATES.NORMAL ? study_normal : null}
        {state === STATES.PHONE ? study_phone : null}
        {state === STATES.STANDING ? study_standing : null}
        {state === STATES.END ? study_end : null}
      </div>
      <Footer />
    </>
  );
}
